# Simulation from BGvM2 model.

# --------------------------------------------------------------------------------------------------------------
# Normalizing constant for BgvM_2 model
# --------------------------------------------------------------------------------------------------------------

library(pracma)

bGvMcnst <- function(par){
intgrnd <- function(x,y){
exp(par[1]*cos(x-par[5])+par[2]*cos(y-par[6])+par[3]*cos(2*(x-par[7]))+par[4]*cos(2*(y-par[8]))+par[9]*cos(x-y-par[11])+par[10]*cos(x+y-par[12]))
}
integral2(intgrnd, 0, 2*pi, 0, 2*pi, reltol = 1e-10)$Q
}

# --------------------------------------------------------------------------------------------------------------
# Density for BgvM_2 model
# --------------------------------------------------------------------------------------------------------------

bGvMden <- function(x,y,par,cnst){
exp(par[1]*cos(x-par[5])+par[2]*cos(y-par[6])+par[3]*cos(2*(x-par[7]))+par[4]*cos(2*(y-par[8]))+par[9]*cos(x-y-par[11])+par[10]*cos(x+y-par[12]))/cnst
}


# --------------------------------------------------------------------------------------------------------------
# Parameter values and density
# --------------------------------------------------------------------------------------------------------------

kap11 <- 1.9 ; kap12 <- 1 ; kap21 <- 0.3 ; kap22 <- -0.7 
mu11 <- 0 ; mu12 <- -0.3 ; mu21 <- -0.9 ; mu22 <- 0.9  
u <- -0.7 ; v <- 0.8 ; alpha <- 1.3 ; beta <- 0

par <- c(kap11, kap12, kap21, kap22, mu11, mu12, mu21, mu22, u, v, alpha, beta) ; ncnst <- bGvMcnst(par)

thet1 <- seq(-pi, pi, 2*pi/800) ; thet2 <- seq(-pi, pi, 2*pi/800)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- bGvMden(t1, t2, par, ncnst)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/800), seq(-pi, pi, 2*pi/800), matrix(cop,801,801), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=30, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
#points(theta1, theta2, pch=16, cex=0.6, col="red")

maxden <- max(cop) ; maxden

# --------------------------------------------------------------------------------------------------------------
# Simulation (simple acceptance-rejection)
# --------------------------------------------------------------------------------------------------------------
n <- 2000

BgvMSim <- function(par, maxden, ncnst, n) {
theta1 <- 0 ; theta2 <- 0
for (i in 1:n) {
stopgo <- 1
while (stopgo == 1) {
t1 <- 2*pi*runif(1,0,1)-pi ; t2 <- 2*pi*runif(1,0,1)-pi
boxh <- maxden*runif(1,0,1)
dval <- bGvMden(t1,t2,par,ncnst)
if (boxh <= dval) {
stopgo <- 0
theta1[i] <- t1 ; theta2[i] <- t2
} } } 
return(list(theta1,theta2))
}

set.seed(739271)

ptm <- proc.time()
SimRes <- BgvMSim(par, maxden, ncnst, n)
proc.time()-ptm

theta1 <- SimRes[[1]] ; theta2 <- SimRes[[2]] 
points(theta1, theta2, pch=16, cex=0.6, col="red")

# rm(list=setdiff(ls(), c("theta1", "theta2")))

