# Simulation from same BNNTS(1,2) model as the BNNTS(1,2) model fitted to the
# simulated data from the BGvM2 model.

# Makes use of the data in the SimDatBGvM2 workspace to fit a BNNTS(1,2) model
# and obtain its ML fitted coefficients.

library(CircNNTSR)

y <- cbind(theta1,theta2)

M <- c(1,2) ; R <- length(M)
est<-mnntsmanifoldnewtonestimation(y,M,R,100) ; est

cest <- est$cestimates
ccoef <- est$cestimates[,3]

thet1 <- seq(-pi, pi, 2*pi/800) ; thet2 <- seq(-pi, pi, 2*pi/800)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cord <- cbind(t1,t2)
cop[copind] <- mnntsdensity(cord,ccoef,M,R)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/800), seq(-pi, pi, 2*pi/800), matrix(cop,801,801), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=15, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))

ptm <- proc.time()
maxden <- max(cop) ; n <- 2000 ; set.seed(28945364)
theta1 <- 0 ; theta2 <- 0
for (i in 1:n) {
stopgo <- 1
while (stopgo == 1) {
t1 <- 2*pi*runif(1,0,1)-pi ; t2 <- 2*pi*runif(1,0,1)-pi
boxh <- maxden*runif(1,0,1)
cord <- cbind(t1,t2)
dval <- mnntsdensity(cord,ccoef,M,R)
if (boxh <= dval) {
stopgo <- 0
theta1[i] <- t1 ; theta2[i] <- t2
} } } 
proc.time()-ptm

points(theta1, theta2, pch=16, cex=0.6, col="red")

# rm(list=setdiff(ls(), c("theta1", "theta2")))

