# Uses data in SimDatBNNTS(1,2) workspace

y <- cbind(theta1,theta2)

# --------------------------------------------------------------------------------------------------------------
# Fitting sine model
# --------------------------------------------------------------------------------------------------------------

## Log-like function for Sine model 

minusloglikS <- function(x,y){
n <- nrow(y)
loglik <- sum(x[1]*cos(y[,1]-x[4]) + x[2]*cos(y[,2]-x[5]) + 
x[3]*sin(y[,1]-x[4])*sin(y[,2]-x[5])) - n*log(biVMSC(x[1:3]))
minusloglikS <- -loglik
return (minusloglikS)
}

## Normalizing constant for Sine model

biVMSC <- function(kvec, a = 0, b = 2 * pi, tol = .Machine$double.eps^0.5) {
integrand <- function(x)
{k23<-sqrt(kvec[2]^2+(kvec[3]^2)*(sin(x))^2)
2*pi*besselI(k23,0)*exp(kvec[1]*cos(x))}
integrate(integrand, lower = a, upper = b)$value
}

start <- c(1.5,2,1,0,pi/8)

try <- optim(par=start,fn=minusloglikS,method="L-BFGS-B",y=y,lower=c(0,0,-Inf,-pi,-pi),upper=c(Inf,Inf,Inf,pi,pi),control=list(trace=T),hessian=T)
try$par
# [1]  1.11175325  1.32103382 -0.34296799 -0.09074893 -0.08921149
try$value
# [1] 6183.437

npar <- 5 ; maxll <- -try$value ; n <- nrow(y)
AIC <- 2*npar-2*maxll ; AIC
# [1] 12376.87
BIC <- npar*log(n)-2*maxll ; BIC
# [1] 12404.88

parML <- try$par ; cnstML <- biVMSC(parML)

biVMSden <- function(x,y,par,cnst){
exp(par[1]*cos(x-par[4])+par[2]*cos(y-par[5])+par[3]*sin(x-par[4])*sin(y-par[5]))/cnst
}

thet1 <- seq(-pi, pi, 2*pi/200) ; thet2 <- seq(-pi, pi, 2*pi/200)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- biVMSden(t1, t2, parML, cnstML)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=20, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
# contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", levels=c(0.025,0.05,0.1,0.2,0.5,1,1.5,2), cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(theta1, theta2, pch=16, cex=0.6, col="red")

# --------------------------------------------------------------------------------------------------------------
# Fitting sine-skewed Sine model
# --------------------------------------------------------------------------------------------------------------

## Log-like function for sine-skewed Sine model 

minusloglikSSS <- function(x,y){
n <- nrow(y)
mult <- x[7] ; maxlam2 <- 1-abs(x[6]) ; lam2 <- -maxlam2+mult*2*maxlam2
loglik <- sum(x[1]*cos(y[,1]-x[4]) + x[2]*cos(y[,2]-x[5]) + x[3]*sin(y[,1]-x[4])*sin(y[,2]-x[5])) - n*log(biVMSC(x[1:3])) + sum(log(1+x[6]*sin(y[,1]-x[4])+lam2*sin(y[,2]-x[5]))) 
minusloglik <- -loglik
return (minusloglik)
}

start <- c(1,1,0.4,0,pi/4,0,0.5)

try <- optim(par=start,fn=minusloglikSSS,method="L-BFGS-B",y=y,lower=c(0,0,-Inf,-pi,-pi,-1,0),upper=c(Inf,Inf,Inf,pi,pi,1,1),control=list(trace=T),hessian=T)
try$par
# [1]  1.111730857  1.314900276 -0.343550089 -0.099425017 -0.010230397
# [6] -0.005759056  0.446436765

try$value
# [1] 6183.315

npar <- 7 ; maxll <- -try$value ; n <- nrow(y)
AIC <- 2*npar-2*maxll ; AIC
# [1] 12380.63
BIC <- npar*log(n)-2*maxll ; BIC
# [1] 12419.84

parML <- try$par ; cnstML <- biVMSC(parML)
maxlam2 <- 1-abs(parML[6]) ; parML[7] <- -maxlam2+parML[7]*2*maxlam2
parML
# [1]  1.111730857  1.314900276 -0.343550089 -0.099425017 -0.010230397
# [6] -0.005759056 -0.106509523

biVMSSSden <- function(x,y,par,cnst){
(1+par[6]*sin(x-par[4])+par[7]*sin(y-par[5]))*exp(par[1]*cos(x-par[4])+par[2]*cos(y-par[5])+par[3]*sin(x-par[4])*sin(y-par[5]))/cnst
}

thet1 <- seq(-pi, pi, 2*pi/200) ; thet2 <- seq(-pi, pi, 2*pi/200)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- biVMSSSden(t1, t2, parML, cnstML)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=20, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
#contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", levels=c(0.025,0.05,0.1,0.2,0.5,1,1.5,2), cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(theta1, theta2, pch=16, cex=0.6, col="red")

# --------------------------------------------------------------------------------------------------------------
# Fitting cosine model (positive interaction)
# --------------------------------------------------------------------------------------------------------------

## Log-like function for Cosine model 

minusloglikNC <- function(x, y){n=nrow(y)
loglik<-sum(x[1]*cos(y[,1]-x[4])+x[2]*cos(y[,2]-x[5])-x[3]*cos(y[,1]-y[,2]+ 
x[5]-x[4]))-n*log(biVMC(x[1:3]))
minusloglikNC =  -loglik
return(minusloglikNC)
}

### Normalizing constant for Cosine model

biVMC <- function(kvec,a=0,b=2*pi,tol=.Machine$double.eps^0.5){
integrand <- function(x){
k13<-sqrt(kvec[1]^2+kvec[3]^2-2*kvec[1]*kvec[3]*cos(x))
2*pi*besselI(k13,0)*exp(kvec[2]*cos(x))}
integrate(integrand, lower = a, upper = b)$value}

start <- c(1,1,1,0,pi/4)

try <- optim(par=start,fn=minusloglikNC,method="L-BFGS-B",y=y,lower=c(0,0,-Inf,-pi,-pi),upper=c(Inf,Inf,Inf,pi,pi),control=list(trace=T),hessian=T)
try$par
# [1]  1.19911888  1.39709225  0.16510430 -0.09221014 -0.09030785
try$value
# [1] 6197.394

npar <- 5 ; maxll <- -try$value ; n <- nrow(y)
AIC <- 2*npar-2*maxll ; AIC
# [1] 12404.79
BIC <- npar*log(n)-2*maxll ; BIC
# [1] 12432.79

parML <- try$par ; cnstML <- biVMC(parML)

biVMCden <- function(x,y,par,cnst){
exp(par[1]*cos(x-par[4])+par[2]*cos(y-par[5])-par[3]*cos((x-par[4])-(y-par[5])))/cnst
}

thet1 <- seq(-pi, pi, 2*pi/200) ; thet2 <- seq(-pi, pi, 2*pi/200)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- biVMCden(t1, t2, parML, cnstML)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=15, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
#contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", levels=c(0.025,0.05,0.1,0.2,0.5,1,1.5,2), cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(theta1, theta2, pch=16, cex=0.6, col="red")

# --------------------------------------------------------------------------------------------------------------
# Fitting cosine model (negative interaction)
# --------------------------------------------------------------------------------------------------------------

## Log-like function for Cosine model 

minusloglikNCN <- function(x, y){n=nrow(y)
loglik<-sum(x[1]*cos(y[,1]-x[4])+x[2]*cos(y[,2]-x[5])-x[3]*cos(y[,1]+y[,2]- 
x[5]-x[4]))-n*log(biVMC(x[1:3]))
minusloglikNCN =  -loglik
return(minusloglikNCN)
}

start <- c(1,1,1,0,pi/4)

try <- optim(par=start,fn=minusloglikNCN,method="L-BFGS-B",y=y,lower=c(0,0,-Inf,-pi,-pi),upper=c(Inf,Inf,Inf,pi,pi),control=list(trace=T),hessian=T)
try$par
# [1]  0.96376757  1.19264426 -0.28024517 -0.08949476 -0.08879269
try$value
# [1] 6182.173

npar <- 5 ; maxll <- -try$value ; n <- nrow(y)
AIC <- 2*npar-2*maxll ; AIC
# [1] 12374.35
BIC <- npar*log(n)-2*maxll ; BIC
# [1] 12402.35

parML <- try$par ; cnstML <- biVMC(parML)

biVMCNden <- function(x,y,par,cnst){
exp(par[1]*cos(x-par[4])+par[2]*cos(y-par[5])-par[3]*cos((x-par[4])+(y-par[5])))/cnst
}

thet1 <- seq(-pi, pi, 2*pi/200) ; thet2 <- seq(-pi, pi, 2*pi/200)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- biVMCNden(t1, t2, parML, cnstML)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=15, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
#contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", levels=c(0.025,0.05,0.1,0.2,0.5,1,1.5,2), cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(theta1, theta2, pch=16, cex=0.6, col="red")

# --------------------------------------------------------------------------------------------------------------
# Fitting Hybrid model 
# --------------------------------------------------------------------------------------------------------------

## Log-like function for Hybrid model 

minusloglikHyb <- function(x, y){n=nrow(y)
loglik<-sum(x[1]*cos(y[,1]-x[4])+x[2]*cos(y[,2]-x[5])+cosh(x[3]-1)*cos(y[,1]-x[4])*cos(y[,2]-x[5])+sinh(x[3])*sin(y[,1]-x[4])*sin(y[,2]-x[5]))-n*log(biVMHyb(x[1:3]))
minusloglik =  -loglik
return(minusloglik)
}

library(pracma)

### Normalizing constant for Hybrid model

biVMHyb <- function(par,a=0,b=2*pi,tol=.Machine$double.eps^0.5){
intgrnd <- function(x,y){
exp(par[1]*cos(x)+par[2]*cos(y)+cosh(par[3]-1)*cos(x)*cos(y)+sinh(par[3])*sin(x)*sin(y))
}
integral2(intgrnd, a, b, a, b, reltol = 1e-10)$Q
}

start <- c(1,1,-2,-pi/4,pi/4)

try <- optim(par=start,fn=minusloglikHyb,method="L-BFGS-B",y=y,lower=c(0,0,-Inf,-pi,-pi),upper=c(Inf,Inf,Inf,pi,pi),control=list(trace=T),hessian=T)
try$par
# [1]  0.55219144  0.90557504  0.25760858 -0.08624668 -0.08804873
try$value
# [1] 6389.959

npar <- 5 ; maxll <- -try$value ; n <- nrow(y)
AIC <- 2*npar-2*maxll ; AIC
# [1] 12789.92
BIC <- npar*log(n)-2*maxll ; BIC
# [1] 12817.92

parML <- try$par ; cnstML <- biVMHyb(parML)

biVMHybden <- function(x,y,par,cnst){
exp(par[1]*cos(x-par[4])+par[2]*cos(y-par[5])+cosh(par[3]-1)*cos(x-par[4])*cos(y-par[5])+sinh(par[3])*sin(x-par[4])*sin(y-par[5]))/cnst
}

thet1 <- seq(-pi, pi, 2*pi/200) ; thet2 <- seq(-pi, pi, 2*pi/200)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- biVMHybden(t1, t2, parML, cnstML)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=15, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
#contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", levels=seq(0.01,29,0.02), cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(theta1, theta2, pch=16, cex=0.6, col="red")


# --------------------------------------------------------------------------------------------------------------
# Fitting Rivest model (6 parameters)
# --------------------------------------------------------------------------------------------------------------

## Log-like function for Rivest model 

minusloglikRiv <- function(x, y){
n=nrow(y)
loglik<-sum(x[1]*cos(y[,1]-x[5])+x[2]*cos(y[,2]-x[6])+x[3]*cos(y[,1]-x[5])*cos(y[,2]-x[6])+x[4]*sin(y[,1]-x[5])*sin(y[,2]-x[6]))-n*log(biVMRiv(x[1:4]))
minusloglik =  -loglik
return(minusloglik)
}

library(pracma)

### Normalizing constant for Rivest model

biVMRiv <- function(par,a=0,b=2*pi,tol=.Machine$double.eps^0.5){
intgrnd <- function(x,y){
exp(par[1]*cos(x)+par[2]*cos(y)+par[3]*cos(x)*cos(y)+par[4]*sin(x)*sin(y))
}
integral2(intgrnd, a, b, a, b, reltol = 1e-10)$Q
}

# start <- c(1,1,1,1,0,pi/4)

start <- c(2,2,-1,3,0,pi/4)

try <- optim(par=start,fn=minusloglikRiv,method="L-BFGS-B",y=y,lower=c(0,0,-Inf,-Inf,-pi,-pi),upper=c(Inf,Inf,Inf,Inf,pi,pi),control=list(trace=T),hessian=T)
try$par
# [1]  1.01864800  1.24002036  0.17551355 -0.34055087 -0.08986069 -0.08883650
try$value
# [1] 6180.403

npar <- 6 ; maxll <- -try$value ; n <- nrow(y)
AIC <- 2*npar-2*maxll ; AIC
# [1] 12372.81
BIC <- npar*log(n)-2*maxll ; BIC
# [1] 12406.41

parML <- try$par ; cnstML <- biVMRiv(parML)

biVMRivden <- function(x,y,par,cnst){
exp(par[1]*cos(x-par[5])+par[2]*cos(y-par[6])+par[3]*cos(x-par[5])*cos(y-par[6])+par[4]*sin(x-par[5])*sin(y-par[6]))/cnst
}

thet1 <- seq(-pi, pi, 2*pi/200) ; thet2 <- seq(-pi, pi, 2*pi/200)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- biVMRivden(t1, t2, parML, cnstML)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=20, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
#contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", levels=seq(0.01,0.28,0.02), cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(theta1, theta2, pch=16, cex=0.6, col="red")

# --------------------------------------------------------------------------------------------------------------
# Fitting Full BvM model (8 parameters)
# --------------------------------------------------------------------------------------------------------------

## Log-like function for BvM model 

minusloglikBvM <- function(x, y){n=nrow(y)
loglik<-sum(x[1]*cos(y[,1]-x[7])+x[2]*cos(y[,2]-x[8])+x[3]*cos(y[,1]-x[7])*cos(y[,2]-x[8])+x[6]*sin(y[,1]-x[7])*sin(y[,2]-x[8])
+x[5]*sin(y[,1]-x[7])*cos(y[,2]-x[8])+x[4]*cos(y[,1]-x[7])*sin(y[,2]-x[8]))-n*log(biVMCon(x[1:6]))
minusloglik =  -loglik
return(minusloglik)
}

library(pracma)

### Normalizing constant for BvM model

biVMCon <- function(par,a=0,b=2*pi,tol=.Machine$double.eps^0.5){
intgrnd <- function(x,y){
exp(par[1]*cos(x)+par[2]*cos(y)+par[3]*cos(x)*cos(y)+par[6]*sin(x)*sin(y)+par[5]*sin(x)*cos(y)+par[4]*cos(x)*sin(y))
}
integral2(intgrnd, a, b, a, b, reltol = 1e-10)$Q
}

start <- c(1.5,2,0.5,-0.5,0.7,-2,0,pi/8)
#start <- try$par

try <- optim(par=start,fn=minusloglikBvM,method="L-BFGS-B",y=y,lower=c(0,0,-Inf,-Inf,-Inf,-Inf,-pi,-pi),upper=c(Inf,Inf,Inf,Inf,Inf,Inf,pi,pi),control=list(trace=T),hessian=T)
try$par
# [1]  1.01882328  1.24018779  0.17491868  0.02288545  0.03047860 -0.34002841
# [7] -0.10598734 -0.09765560
try$value
# [1] 6180.289

npar <- 8 ; maxll <- -try$value ; n <- nrow(y)
AIC <- 2*npar-2*maxll ; AIC
# [1] 12376.58
BIC <- npar*log(n)-2*maxll ; BIC
# [1] 12421.38

parML <- try$par ; cnstML <- biVMCon(parML)

biVMden <- function(x,y,par,cnst){
exp(par[1]*cos(x-par[7])+par[2]*cos(y-par[8])+par[3]*cos(x-par[7])*cos(y-par[8])+par[6]*sin(x-par[7])*sin(y-par[8])+par[5]*sin(x-par[7])*cos(y-par[8])+par[4]*cos(x-par[7])*sin(y-par[8]))/cnst
}

thet1 <- seq(-pi, pi, 2*pi/200) ; thet2 <- seq(-pi, pi, 2*pi/200)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- biVMden(t1, t2, parML, cnstML)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=20, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
#contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", levels=c(0.01,0.05,0.1,0.2,0.5,1,1.5,2), cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(theta1, theta2, pch=16, cex=0.6, col="red")


