# Simulation from full BvM model.

# --------------------------------------------------------------------------------------------------------------
# Normalizing constant for Full BvM model
# --------------------------------------------------------------------------------------------------------------

library(pracma)

biVMCon <- function(par,a=0,b=2*pi,tol=.Machine$double.eps^0.5){
intgrnd <- function(x,y){
exp(par[1]*cos(x)+par[2]*cos(y)+par[3]*cos(x)*cos(y)+par[6]*sin(x)*sin(y)+par[5]*sin(x)*cos(y)+par[4]*cos(x)*sin(y))
}
integral2(intgrnd, a, b, a, b, reltol = 1e-10)$Q
}

# --------------------------------------------------------------------------------------------------------------
# Density for Full BvM model
# --------------------------------------------------------------------------------------------------------------

biVMden <- function(x,y,par,cnst){
exp(par[1]*cos(x-par[7])+par[2]*cos(y-par[8])+par[3]*cos(x-par[7])*cos(y-par[8])+par[6]*sin(x-par[7])*sin(y-par[8])+par[5]*sin(x-par[7])*cos(y-par[8])+par[4]*cos(x-par[7])*sin(y-par[8]))/cnst
}

# --------------------------------------------------------------------------------------------------------------
# Parameter values and density
# --------------------------------------------------------------------------------------------------------------

# Original parameter values used for simulated sample.
# kap1 <- 1.5 ; kap2 <- 2 ; mu1 <- 0 ; mu2 <- pi/8 
# a11 <- 0.5 ; a12 <- -0.5 ; a21 <- 0.7 ; a22 <- -2

# Second attempt
# kap1 <- 1.5 ; kap2 <- 2 ; mu1 <- 0 ; mu2 <- pi/8 
# a11 <- 0.5 ; a12 <- 0.5 ; a21 <- -0.5 ; a22 <- -1

# Final parameter values used for simulated sample.
kap1 <- 1.8 ; kap2 <- 1 ; mu1 <- 0 ; mu2 <- -pi/16 
a11 <- 0.5 ; a12 <- 0.6 ; a21 <- -0.5 ; a22 <- -1

par <- c(kap1, kap2, a11, a12, a21, a22, mu1, mu2) ; ncnst <- biVMCon(par)

thet1 <- seq(-pi, pi, 2*pi/800) ; thet2 <- seq(-pi, pi, 2*pi/800)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- biVMden(t1, t2, par, ncnst)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/800), seq(-pi, pi, 2*pi/800), matrix(cop,801,801), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=30, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
#points(theta1, theta2, pch=16, cex=0.6, col="red")

maxden <- max(cop) ; maxden

# --------------------------------------------------------------------------------------------------------------
# Simulation (simple acceptance-rejection)
# --------------------------------------------------------------------------------------------------------------
n <- 2000

BvMSim <- function(par, maxden, ncnst, n) {
theta1 <- 0 ; theta2 <- 0
for (i in 1:n) {
stopgo <- 1
while (stopgo == 1) {
t1 <- 2*pi*runif(1,0,1)-pi ; t2 <- 2*pi*runif(1,0,1)-pi
boxh <- maxden*runif(1,0,1)
dval <- biVMden(t1,t2,par,ncnst)
if (boxh <= dval) {
stopgo <- 0
theta1[i] <- t1 ; theta2[i] <- t2
} } } 
return(list(theta1,theta2))
}

set.seed(2123428)

ptm <- proc.time()
SimRes <- BvMSim(par, maxden, ncnst, n)
proc.time()-ptm

theta1 <- SimRes[[1]] ; theta2 <- SimRes[[2]] 
points(theta1, theta2, pch=16, cex=0.6, col="red")

# rm(list=setdiff(ls(), c("theta1", "theta2")))

