# Simulation from Rivest model.

library(pracma)

### Normalizing constant for Rivest model

biVMRiv <- function(par,a=-pi,b=pi,tol=.Machine$double.eps^0.5){
intgrnd <- function(x,y){
exp(par[1]*cos(x)+par[2]*cos(y)+par[3]*cos(x)*cos(y)+par[4]*sin(x)*sin(y))
}
integral2(intgrnd, a, b, a, b, reltol = 1e-10)$Q
}

## Density of Hybrid model

biVMRivden <- function(x,y,par,cnst){
exp(par[1]*cos(x-par[5])+par[2]*cos(y-par[6])+par[3]*cos(x-par[5])*cos(y-par[6])+par[4]*sin(x-par[5])*sin(y-par[6]))/cnst
}

# --------------------------------------------------------------------------------------------------------------
# Parameter values and density
# --------------------------------------------------------------------------------------------------------------

kap1 <- 2 ; kap2 <- 1 ; alpha <- -3 ; beta <- 2 ; mu1 <- 0 ; mu2 <- pi

par <- c(kap1, kap2, alpha, beta, mu1, mu2) ; ncnst <- biVMRiv(par)

thet1 <- seq(-pi, pi, 2*pi/800) ; thet2 <- seq(-pi, pi, 2*pi/800)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- biVMRivden(t1, t2, par, ncnst)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/800), seq(-pi, pi, 2*pi/800), matrix(cop,801,801), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=30, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))

maxden <- max(cop) ; maxden

# --------------------------------------------------------------------------------------------------------------
# Simulation (simple acceptance-rejection)
# --------------------------------------------------------------------------------------------------------------

n <- 2000

BvMRivSim <- function(par, maxden, ncnst, n) {
theta1 <- 0 ; theta2 <- 0
for (i in 1:n) {
stopgo <- 1
while (stopgo == 1) {
t1 <- 2*pi*runif(1,0,1)-pi ; t2 <- 2*pi*runif(1,0,1)-pi
boxh <- maxden*runif(1,0,1)
dval <- biVMRivden(t1,t2,par,ncnst)
if (boxh <= dval) {
stopgo <- 0
theta1[i] <- t1 ; theta2[i] <- t2
} } } 
return(list(theta1,theta2))
}

set.seed(63001256)

ptm <- proc.time()
SimRes <- BvMRivSim(par, maxden, ncnst, n)
proc.time()-ptm

theta1 <- SimRes[[1]] ; theta2 <- SimRes[[2]] 
points(theta1, theta2, pch=16, cex=0.6, col="red")

# rm(list=setdiff(ls(), c("theta1", "theta2")))

