# Uses data in sine-skewed-Sine_SimDat workspace

# Data simulated from sine-skewed Sine model with
# kap1 <- 1 ; kap2 <- 1.5 ; lambda <- 1; mu1 <- 0 ; mu2 <- pi/8 ; lambda1 <- -0.5 ; lambda2 <- 0.5 

y <- cbind(theta1,theta2)

# --------------------------------------------------------------------------------------------------------------
# Fitting sine model
# --------------------------------------------------------------------------------------------------------------

## Log-like function for Sine model 

minusloglikS <- function(x,y){
n <- nrow(y)
loglik <- sum(x[1]*cos(y[,1]-x[4]) + x[2]*cos(y[,2]-x[5]) + 
x[3]*sin(y[,1]-x[4])*sin(y[,2]-x[5])) - n*log(biVMSC(x[1:3]))
minusloglikS <- -loglik
return (minusloglikS)
}

## Normalizing constant for Sine model

biVMSC <- function(kvec, a = 0, b = 2 * pi, tol = .Machine$double.eps^0.5) {
integrand <- function(x)
{k23<-sqrt(kvec[2]^2+(kvec[3]^2)*(sin(x))^2)
2*pi*besselI(k23,0)*exp(kvec[1]*cos(x))}
integrate(integrand, lower = a, upper = b)$value
}

start <- c(1,1.5,1,0,pi/8)

try <- optim(par=start,fn=minusloglikS,method="L-BFGS-B",y=y,lower=c(0,0,-Inf,-pi,-pi),upper=c(Inf,Inf,Inf,pi,pi),control=list(trace=T),hessian=T)
try$par
# [1]  1.068607  1.592995  1.009356 -0.231472  0.627945
try$value
# [1] 5957.156

npar <- 5 ; maxll <- -try$value ; n <- nrow(y)
AIC <- 2*npar-2*maxll ; AIC
# [1] 11924.31
BIC <- npar*log(n)-2*maxll ; BIC
# [1] 11952.32

parML <- try$par ; cnstML <- biVMSC(parML)

biVMSden <- function(x,y,par,cnst){
exp(par[1]*cos(x-par[4])+par[2]*cos(y-par[5])+par[3]*sin(x-par[4])*sin(y-par[5]))/cnst
}

thet1 <- seq(-pi, pi, 2*pi/200) ; thet2 <- seq(-pi, pi, 2*pi/200)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- biVMSden(t1, t2, parML, cnstML)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=15, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
#contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", levels=c(0.025,0.05,0.1,0.2,0.5,1,1.5,2), cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(theta1, theta2, pch=16, cex=0.6, col="red")

# --------------------------------------------------------------------------------------------------------------
# Fitting sine-skewed Sine model
# --------------------------------------------------------------------------------------------------------------

## Log-like function for sine-skewed Sine model 

minusloglikSSS <- function(x,y){
n <- nrow(y)
mult <- x[7] ; maxlam2 <- 1-abs(x[6]) ; lam2 <- -maxlam2+mult*2*maxlam2
loglik <- sum(x[1]*cos(y[,1]-x[4]) + x[2]*cos(y[,2]-x[5]) + x[3]*sin(y[,1]-x[4])*sin(y[,2]-x[5])) - n*log(biVMSC(x[1:3])) + sum(log(1+x[6]*sin(y[,1]-x[4])+lam2*sin(y[,2]-x[5]))) 
minusloglik <- -loglik
return (minusloglik)
}

start <- c(1,1.5,1,0,pi/8,-0.49,0.49)

try <- optim(par=start,fn=minusloglikSSS,method="L-BFGS-B",y=y,lower=c(0,0,-Inf,-pi,-pi,-1,0),upper=c(Inf,Inf,Inf,pi,pi,1,1),control=list(trace=T),hessian=T)
try$par
# [1]  0.98167774  1.50990215  0.88553740  0.04016894  0.40668576 -0.49436084 1.00000000
try$value
# [1] 5923.497

npar <- 7 ; maxll <- -try$value ; n <- nrow(y)
AIC <- 2*npar-2*maxll ; AIC
# [1] 11860.99
BIC <- npar*log(n)-2*maxll ; BIC
# [1] 11900.2

parML <- try$par ; cnstML <- biVMSC(parML)
maxlam2 <- 1-abs(parML[6]) ; parML[7] <- -maxlam2+parML[7]*2*maxlam2
parML
# [1]  0.98167774  1.50990215  0.88553740  0.04016894  0.40668576 -0.49436084 0.50563916

biVMSSSden <- function(x,y,par,cnst){
(1+par[6]*sin(x-par[4])+par[7]*sin(y-par[5]))*exp(par[1]*cos(x-par[4])+par[2]*cos(y-par[5])+par[3]*sin(x-par[4])*sin(y-par[5]))/cnst
}

thet1 <- seq(-pi, pi, 2*pi/200) ; thet2 <- seq(-pi, pi, 2*pi/200)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- biVMSSSden(t1, t2, parML, cnstML)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=15, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
#contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", levels=c(0.025,0.05,0.1,0.2,0.5,1,1.5,2), cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(theta1, theta2, pch=16, cex=0.6, col="red")

# --------------------------------------------------------------------------------------------------------------
# Fitting cosine model (positive interaction)
# --------------------------------------------------------------------------------------------------------------

## Log-like function for Cosine model 

minusloglikNC <- function(x, y){n=nrow(y)
loglik<-sum(x[1]*cos(y[,1]-x[4])+x[2]*cos(y[,2]-x[5])-x[3]*cos(y[,1]-y[,2]+ 
x[5]-x[4]))-n*log(biVMC(x[1:3]))
minusloglikNC =  -loglik
return(minusloglikNC)
}

### Normalizing constant for Cosine model

biVMC <- function(kvec,a=0,b=2*pi,tol=.Machine$double.eps^0.5){
integrand <- function(x){
k13<-sqrt(kvec[1]^2+kvec[3]^2-2*kvec[1]*kvec[3]*cos(x))
2*pi*besselI(k13,0)*exp(kvec[2]*cos(x))}
integrate(integrand, lower = a, upper = b)$value}

start <- c(1,1.5,1,0,pi/8)

try <- optim(par=start,fn=minusloglikNC,method="L-BFGS-B",y=y,lower=c(0,0,-Inf,-pi,-pi),upper=c(Inf,Inf,Inf,pi,pi),control=list(trace=T),hessian=T)
try$par
# [[1]  0.6992608  1.3111896 -0.6082861 -0.2403792  0.6050347
try$value
# [1] 6025.837

npar <- 5 ; maxll <- -try$value ; n <- nrow(y)
AIC <- 2*npar-2*maxll ; AIC
# [1] 12061.67
BIC <- npar*log(n)-2*maxll ; BIC
# [1] 12089.68

parML <- try$par ; cnstML <- biVMC(parML)

biVMCden <- function(x,y,par,cnst){
exp(par[1]*cos(x-par[4])+par[2]*cos(y-par[5])-par[3]*cos((x-par[4])-(y-par[5])))/cnst
}

thet1 <- seq(-pi, pi, 2*pi/200) ; thet2 <- seq(-pi, pi, 2*pi/200)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- biVMCden(t1, t2, parML, cnstML)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=15, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
#contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", levels=c(0.025,0.05,0.1,0.2,0.5,1,1.5,2), cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(theta1, theta2, pch=16, cex=0.6, col="red")

# --------------------------------------------------------------------------------------------------------------
# Fitting cosine model (negative interaction)
# --------------------------------------------------------------------------------------------------------------

## Log-like function for Cosine model 

minusloglikNCN <- function(x, y){n=nrow(y)
loglik<-sum(x[1]*cos(y[,1]-x[4])+x[2]*cos(y[,2]-x[5])-x[3]*cos(y[,1]+y[,2]- 
x[5]-x[4]))-n*log(biVMC(x[1:3]))
minusloglikNCN =  -loglik
return(minusloglikNCN)
}

start <- c(1,1.5,1,-0,pi/8)

try <- optim(par=start,fn=minusloglikNCN,method="L-BFGS-B",y=y,lower=c(0,0,-Inf,-pi,-pi),upper=c(Inf,Inf,Inf,pi,pi),control=list(trace=T),hessian=T)
try$par
# [1]  1.5733672  1.9908856  0.7950672 -0.2745977  0.6536648

try$value
# [1] 5992.745

npar <- 5 ; maxll <- -try$value ; n <- nrow(y)
AIC <- 2*npar-2*maxll ; AIC
# [1] 11995.49
BIC <- npar*log(n)-2*maxll ; BIC
# [1] 12023.49

parML <- try$par ; cnstML <- biVMC(parML)

biVMCNden <- function(x,y,par,cnst){
exp(par[1]*cos(x-par[4])+par[2]*cos(y-par[5])-par[3]*cos((x-par[4])+(y-par[5])))/cnst
}

thet1 <- seq(-pi, pi, 2*pi/200) ; thet2 <- seq(-pi, pi, 2*pi/200)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- biVMCNden(t1, t2, parML, cnstML)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=15, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
#contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", levels=c(0.025,0.05,0.1,0.2,0.5,1,1.5,2), cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(theta1, theta2, pch=16, cex=0.6, col="red")

# --------------------------------------------------------------------------------------------------------------
# Fitting Hybrid model 
# --------------------------------------------------------------------------------------------------------------

## Log-like function for Hybrid model 

minusloglikHyb <- function(x, y){n=nrow(y)
loglik<-sum(x[1]*cos(y[,1]-x[4])+x[2]*cos(y[,2]-x[5])+cosh(x[3]-1)*cos(y[,1]-x[4])*cos(y[,2]-x[5])+sinh(x[3])*sin(y[,1]-x[4])*sin(y[,2]-x[5]))-n*log(biVMHyb(x[1:3]))
minusloglik =  -loglik
return(minusloglik)
}

library(pracma)

### Normalizing constant for Hybrid model

biVMHyb <- function(par,a=-pi,b=pi,tol=.Machine$double.eps^0.5){
intgrnd <- function(x,y){
exp(par[1]*cos(x)+par[2]*cos(y)+cosh(par[3]-1)*cos(x)*cos(y)+sinh(par[3])*sin(x)*sin(y))
}
integral2(intgrnd, a, b, a, b, reltol = 1e-10)$Q
}

start <- c(1,1.5,1,0,pi/8)

try <- optim(par=start,fn=minusloglikHyb,method="L-BFGS-B",y=y,lower=c(0,0,-Inf,-pi,-pi),upper=c(Inf,Inf,Inf,pi,pi),control=list(trace=T),hessian=T)
try$par
# [1]  0.5320818  1.2853795  0.9264634 -0.2098057  0.6043122

try$value
# [1] 6065.62

npar <- 5 ; maxll <- -try$value ; n <- nrow(y)
AIC <- 2*npar-2*maxll ; AIC
# [1] 12141.24
BIC <- npar*log(n)-2*maxll ; BIC
# [1] 12169.24

parML <- try$par ; cnstML <- biVMHyb(parML)

biVMHybden <- function(x,y,par,cnst){
exp(par[1]*cos(x-par[4])+par[2]*cos(y-par[5])+cosh(par[3]-1)*cos(x-par[4])*cos(y-par[5])+sinh(par[3])*sin(x-par[4])*sin(y-par[5]))/cnst
}

thet1 <- seq(-pi, pi, 2*pi/200) ; thet2 <- seq(-pi, pi, 2*pi/200)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- biVMHybden(t1, t2, parML, cnstML)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
#contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=15, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", levels=seq(0.01,29,0.02), cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(theta1, theta2, pch=16, cex=0.6, col="red")


# --------------------------------------------------------------------------------------------------------------
# Fitting Rivest model (6 parameters)
# --------------------------------------------------------------------------------------------------------------

## Log-like function for Rivest model 

minusloglikRiv <- function(x, y){n=nrow(y)
loglik<-sum(x[1]*cos(y[,1]-x[5])+x[2]*cos(y[,2]-x[6])+x[3]*cos(y[,1]-x[5])*cos(y[,2]-x[6])+x[4]*sin(y[,1]-x[5])*sin(y[,2]-x[6]))-n*log(biVMRiv(x[1:4]))
minusloglik =  -loglik
return(minusloglik)
}

library(pracma)

### Normalizing constant for Rivest model

biVMRiv <- function(par,a=-pi,b=pi,tol=.Machine$double.eps^0.5){
intgrnd <- function(x,y){
exp(par[1]*cos(x)+par[2]*cos(y)+par[3]*cos(x)*cos(y)+par[4]*sin(x)*sin(y))
}
integral2(intgrnd, a, b, a, b, reltol = 1e-10)$Q
}

start <- c(1,1.5,1,-1,0,pi/8)

try <- optim(par=start,fn=minusloglikRiv,method="L-BFGS-B",y=y,lower=c(0,0,-Inf,-Inf,-pi,-pi),upper=c(Inf,Inf,Inf,Inf,pi,pi),control=list(trace=T),hessian=T)
try$par
# [1]  1.1599408  1.6633410 -0.1472479  1.0170405 -0.2354054  0.6321160
try$value
# [1] 5955.618

npar <- 6 ; maxll <- -try$value ; n <- nrow(y)
AIC <- 2*npar-2*maxll ; AIC
# [1] 11923.24
BIC <- npar*log(n)-2*maxll ; BIC
# [1] 11956.84

parML <- try$par ; cnstML <- biVMRiv(parML)

biVMRivden <- function(x,y,par,cnst){
exp(par[1]*cos(x-par[5])+par[2]*cos(y-par[6])+par[3]*cos(x-par[5])*cos(y-par[6])+par[4]*sin(x-par[5])*sin(y-par[6]))/cnst
}

thet1 <- seq(-pi, pi, 2*pi/200) ; thet2 <- seq(-pi, pi, 2*pi/200)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- biVMRivden(t1, t2, parML, cnstML)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=15, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
#contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", levels=seq(0.01,0.28,0.02), cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(theta1, theta2, pch=16, cex=0.6, col="red")

# --------------------------------------------------------------------------------------------------------------
# Fitting Full BvM model (8 parameters)
# --------------------------------------------------------------------------------------------------------------

## Log-like function for BvM model 

minusloglikBvM <- function(x, y){n=nrow(y)
loglik<-sum(x[1]*cos(y[,1]-x[7])+x[2]*cos(y[,2]-x[8])+x[3]*cos(y[,1]-x[7])*cos(y[,2]-x[8])+x[6]*sin(y[,1]-x[7])*sin(y[,2]-x[8])
+x[5]*sin(y[,1]-x[7])*cos(y[,2]-x[8])+x[4]*cos(y[,1]-x[7])*sin(y[,2]-x[8]))-n*log(biVMCon(x[1:6]))
minusloglik =  -loglik
return(minusloglik)
}

library(pracma)

### Normalizing constant for BvM model

biVMCon <- function(par,a=0,b=2*pi,tol=.Machine$double.eps^0.5){
intgrnd <- function(x,y){
exp(par[1]*cos(x)+par[2]*cos(y)+par[3]*cos(x)*cos(y)+par[6]*sin(x)*sin(y)+par[5]*sin(x)*cos(y)+par[4]*cos(x)*sin(y))
}
integral2(intgrnd, a, b, a, b, reltol = 1e-10)$Q
}

start <- c(1,1.5,1,-1,1,-1,0,pi/8)
#start <- try$par

try <- optim(par=start,fn=minusloglikBvM,method="L-BFGS-B",y=y,lower=c(0,0,-Inf,-Inf,-Inf,-Inf,-pi,-pi),upper=c(Inf,Inf,Inf,Inf,Inf,Inf,pi,pi),control=list(trace=T),hessian=T)
try$par
# [1]  1.2460472  1.7350336 -0.3381677 -0.5653949  0.3864603  0.8749254 -0.4933001
# [8]  0.7753988

try$value
# [1] 5929.444

npar <- 8 ; maxll <- -try$value ; n <- nrow(y)
AIC <- 2*npar-2*maxll ; AIC
# [1] 11874.89
BIC <- npar*log(n)-2*maxll ; BIC
# [1] 11919.69

parML <- try$par ; cnstML <- biVMCon(parML)

biVMden <- function(x,y,par,cnst){
exp(par[1]*cos(x-par[7])+par[2]*cos(y-par[8])+par[3]*cos(x-par[7])*cos(y-par[8])+par[6]*sin(x-par[7])*sin(y-par[8])+par[5]*sin(x-par[7])*cos(y-par[8])+par[4]*cos(x-par[7])*sin(y-par[8]))/cnst
}

thet1 <- seq(-pi, pi, 2*pi/200) ; thet2 <- seq(-pi, pi, 2*pi/200)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- biVMden(t1, t2, parML, cnstML)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=15, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
#contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", levels=c(0.01,0.05,0.1,0.2,0.5,1,1.5,2), cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(theta1, theta2, pch=16, cex=0.6, col="red")


