# Simulation from sine-skewed Sine model.

## Normalizing constant for Sine model

biVMScon <- function(kvec, a = 0, b = 2 * pi, tol = .Machine$double.eps^0.5) {
integrand <- function(x)
{k23<-sqrt(kvec[2]^2+(kvec[3]^2)*(sin(x))^2)
2*pi*besselI(k23,0)*exp(kvec[1]*cos(x))}
integrate(integrand, lower = a, upper = b)$value
}

## Density of sine-skewed Sine model

biVMSSSden <- function(x,y,par,cnst){
(1+par[6]*sin(x-par[4])+par[7]*sin(y-par[5]))*exp(par[1]*cos(x-par[4])+par[2]*cos(y-par[5])+par[3]*sin(x-par[4])*sin(y-par[5]))/cnst
}

# --------------------------------------------------------------------------------------------------------------
# Parameter values and density
# --------------------------------------------------------------------------------------------------------------

kap1 <- 1 ; kap2 <- 1.5 ; lambda <- 1; mu1 <- 0 ; mu2 <- pi/8 ; lambda1 <- -0.5 ; lambda2 <- 0.5 

par <- c(kap1, kap2, lambda, mu1, mu2, lambda1, lambda2) ; ncnst <- biVMScon(par)

thet1 <- seq(-pi, pi, 2*pi/800) ; thet2 <- seq(-pi, pi, 2*pi/800)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- biVMSSSden(t1, t2, par, ncnst)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/800), seq(-pi, pi, 2*pi/800), matrix(cop,801,801), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=15, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
#points(theta1, theta2, pch=16, cex=0.4, col="red")

maxden <- max(cop) ; maxden

# --------------------------------------------------------------------------------------------------------------
# Simulation (simple acceptance-rejection)
# --------------------------------------------------------------------------------------------------------------

n <- 2000

BvMSSSineSim <- function(par, maxden, ncnst, n) {
theta1 <- 0 ; theta2 <- 0
for (i in 1:n) {
stopgo <- 1
while (stopgo == 1) {
t1 <- 2*pi*runif(1,0,1)-pi ; t2 <- 2*pi*runif(1,0,1)-pi
boxh <- maxden*runif(1,0,1)
dval <- biVMSSSden(t1,t2,par,ncnst)
if (boxh <= dval) {
stopgo <- 0
theta1[i] <- t1 ; theta2[i] <- t2
} } } 
return(list(theta1,theta2))
}

set.seed(2845395)

ptm <- proc.time()
SimRes <- BvMSSSineSim(par, maxden, ncnst, n)
proc.time()-ptm

theta1 <- SimRes[[1]] ; theta2 <- SimRes[[2]] 
points(theta1, theta2, pch=16, cex=0.6, col="red")

# rm(list=setdiff(ls(), c("theta1", "theta2")))

