# Uses data in M2020_04_06_80 workspace

theta <- cbind(theta1,theta2)

library(pracma)

### Normalizing constant
bGvMcnst <- function(par){
intgrnd <- function(x,y){
exp(par[1]*cos(x-par[5])+par[2]*cos(y-par[6])+par[3]*cos(2*(x-par[7]))+par[4]*cos(2*(y-par[8]))+par[9]*cos(x-y-par[11])+par[10]*cos(x+y-par[12]))
}
integral2(intgrnd, 0, 2*pi, 0, 2*pi, reltol = 1e-10)$Q
}

minusloglik <- function(par, y){
n=nrow(y)
loglik <- sum(par[1]*cos(y[,1]-par[5])+par[2]*cos(y[,2]-par[6])+par[3]*cos(2*(y[,1]-par[7]))+par[4]*cos(2*(y[,2]-par[8]))
+ par[9]*cos(y[,1]-y[,2]-par[11])+par[10]*cos(y[,1]+y[,2]-par[12]))
loglik <- loglik-n*log(bGvMcnst(par))
negll <- -loglik
return(negll)
}

pars <- c(1,1,0,0,-2,-2,-1,-1,1,1,-1,-1)

# low <- c(rep(0,4), rep(-pi,4), rep(0,2), rep(-pi,2)) ; upp <- c(rep(Inf,4), rep(pi,4), rep(Inf,2), rep(pi,2))
low <- c(rep(-Inf,4), rep(-pi,4), rep(-Inf,2), rep(-pi,2)) ; upp <- c(rep(Inf,4), rep(pi,4), rep(Inf,2), rep(pi,2))
try <- optim(par=pars,fn=minusloglik,method="L-BFGS-B",y=theta,lower=low,upper=upp,control=list(trace=T,maxit=200),hessian=T)

try$par
#  [1]  0.83415614  0.82456379 -0.74395093  0.61635852 -2.38561556 -0.99857349
#  [7] -1.84553225 -0.23824241  3.90317244 -1.23248408  0.01819759 -0.67423793

-try$value
# [1] -191.6805

npar <- 12 ; maxll <- -try$value ; n <- length(theta1)
AIC <- 2*npar-2*maxll ; AIC
# [1] 407.361
BIC <- npar*log(n)-2*maxll ; BIC
# [1] 435.9453

parML <- try$par ; cnstML <- bGvMcnst(parML)

bGvMden <- function(x,y,par,cnst){
exp(par[1]*cos(x-par[5])+par[2]*cos(y-par[6])+par[3]*cos(2*(x-par[7]))+par[4]*cos(2*(y-par[8]))+par[9]*cos(x-y-par[11])+par[10]*cos(x+y-par[12]))/cnst
}

thet1 <- seq(-pi, pi, 2*pi/200) ; thet2 <- seq(-pi, pi, 2*pi/200)
nt <- length(thet1) ; copind <- 0 ; cop <- 0
for (i in 1:nt) { t2 <- thet2[i] 
for (j in 1:nt) { t1 <- thet1[j] ; copind <- copind+1
cop[copind] <- bGvMden(t1, t2, parML, cnstML)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", nlevels=20, cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
#contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(cop,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xlab=expression(theta[1]), ylab=expression(theta[2]), lty=1, lwd=2, col="darkblue", levels=seq(0.01,0.29,0.01), cex.lab=1.4, labcex=0.8, xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(theta1, theta2, pch=16, cex=1, col="red")

